import { Component, ViewChild, ElementRef, AfterViewInit, Inject } from '@angular/core';
import { chart } from 'highcharts';
import { Highcharts } from 'angular-highcharts';

import { GraphElementComponent } from './graph-element/graph-element.component';

import { graphSeries, Report, ReportService } from './../report.service';
import { Observable } from 'rxjs/Observable';

@Component({
  selector: 'graphs-page',
  templateUrl: './graphs-page.component.html',
  styleUrls: ['./graphs-page.scss']
})

export class GraphsPageComponent implements AfterViewInit {
  @ViewChild(GraphElementComponent)
  private graphComponent: GraphElementComponent;

  series: graphSeries;

  graphOptions: any;
  selectedGraphType: string;
  selectedInterval: string;
  intervalList: Array<string>;
  startDate: Date;
  endDate: Date;
  graphStartDate: Date;

  constructor(@Inject('ReportService') private reportService: ReportService) {
    this.series = new graphSeries();

    this.intervalList = ["Mensal", "Diário"];
    this.selectedInterval = this.intervalList[1];

    this.endDate = new Date();
    this.startDate = new Date();
    this.startDate.setMonth(this.endDate.getMonth() - 1);
    this.graphStartDate = new Date();

    this.setChartOptions("line");
    this.updateChart();
  }

  ngAfterViewInit() {
  }

  private setChartOptions(graphType: string) {
    this.selectedGraphType = graphType;
    /* We need to reset the options to make sure previous graphs don't mess with the current */
    this.graphOptions = {};

    /* Common options */
    this.graphOptions.colors = ['#8e8cc4', '#4f425b', '#c5dcf2'];
    this.graphOptions.subtitle = { text: "Unidade monitorada: Hospital Erasto Gaertner" };
    this.graphOptions.credits = { enabled: false };

    /* Populate graph based on its type */
    if (this.selectedGraphType == "line") {


      this.graphStartDate = new Date(this.series.start_date);
      let interval: number = 24 * 3600 * 1000; // Graph tick interval in milliseconds
      this.graphOptions.chart = { type: 'column' };
      if (this.selectedInterval == "Mensal" || this.selectedInterval == "Anual")
        interval *= 30;
      if (this.selectedInterval == "Anual")
        interval *= 12;

      this.graphOptions.title = { text: "Distribuição dos atendimentos ao longo do tempo" };
      this.graphOptions.yAxis = { title: { text: "% de pacientes" } };
      this.graphOptions.xAxis = {
        type: "datetime",
        tickInterval: interval
      };

      this.graphOptions.tooltip = { pointFormat: '{series.name}: <b>{point.y:.2f}</b>' };
      this.graphOptions.plotOptions = {
        series: {
          stacking: 'normal'
        }
      }

      this.graphOptions.series = [{
        name: "Geral",
        data: this.series.non_sus_percentage_arr,
        pointStart: Date.UTC(this.graphStartDate.getFullYear(), this.graphStartDate.getMonth(), this.graphStartDate.getDate()),
        pointInterval: interval
      }, {
        name: "SUS",
        data: this.series.sus_percentage_arr,
        pointStart: Date.UTC(this.graphStartDate.getFullYear(), this.graphStartDate.getMonth(), this.graphStartDate.getDate()),
        pointInterval: interval
      }
      ];

    } else if (this.selectedGraphType == "pie") {

      this.graphOptions.chart = { type: 'pie' };
      this.graphOptions.title = { text: 'Proporção de atendidos pelo SUS' };
      this.graphOptions.tooltip = { pointFormat: '{series.name}: <b>{point.y}</b>' }
      this.graphOptions.plotOptions = {
        pie: {
          dataLabels: {
            enabled: true,
            format: '<b>{point.name}</b>: {point.percentage:.2f}%'
          }
        }
      }
      this.graphOptions.series = [{
        name: 'Total de atendidos:',
        data: [{
          name: 'Geral',
          y: this.series.non_sus_total
        }, {
          name: 'SUS',
          y: this.series.sus_total
        }]
      }]

    }

  }


  setTypeofGraph(newType: string) {
    this.selectedGraphType = newType;
  }

  public updateChart() {
    /* Retrieve graph data from service */

    this.reportService.getReports(this.selectedInterval, this.startDate, this.endDate)
      .subscribe(res => {
        this.series = new graphSeries(res);
      },
        error => console.log(error),
        () => {
          this.setChartOptions(this.selectedGraphType);
          this.graphComponent.reset(this.graphOptions);
        }
      );
  }

}